'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function codeGen(t, ast) {
  var JSONStringify = t.memberExpression(t.identifier('JSON'), t.identifier('stringify'));
  function genFieldSet(ast) {
    (0, _assert2.default)(ast.type === 'FieldSet');
    var properties = ast.body.map(genNode);
    return t.objectExpression(properties);
  }
  function isConstantValue(value) {
    return value.type === 'String' || value.type === 'Bool' || value.type === 'Number';
  }
  function genField(ast) {
    (0, _assert2.default)(ast.type === 'Field');
    var name = ast.name;
    var value = null;
    if (ast.body) {
      value = genFieldSet(ast.body);
    } else {
      value = t.booleanLiteral(true);
    }
    if (!ast.args || ast.args.length === 0) {
      return t.objectProperty(ast.alias ? t.stringLiteral(name + ' as ' + ast.alias) : t.identifier(name), value);
    } else {
      var _ret = function () {
        var push = function push(str) {
          if (typeof parts[parts.length - 1] === 'string') {
            parts[parts.length - 1] += str;
          } else {
            parts.push(str);
          }
        };

        var args = ast.args.slice().sort(function (a, b) {
          return a.name < b.name ? -1 : 1;
        });
        var parts = [name + '('];

        args.forEach(function (arg, i) {
          (0, _assert2.default)(arg.type === 'Arg');
          if (i !== 0) {
            push(',');
          }
          push(arg.name);
          push(':');
          if (isConstantValue(arg.val)) {
            push((0, _stringify2.default)(arg.val.val));
          } else {
            parts.push(arg.val.val);
          }
        });
        push(')');
        if (ast.alias) {
          push(' as ' + ast.alias);
        }
        if (parts.length === 1) {
          return {
            v: t.objectProperty(t.stringLiteral(parts[0]), value)
          };
        } else {
          return {
            v: t.objectProperty(parts.map(function (part) {
              return typeof part === 'string' ? t.stringLiteral(part) : t.callExpression(JSONStringify, [part]);
            }).reduce(function (left, right) {
              return t.binaryExpression('+', left, right);
            }), value, true // computed
            )
          };
        }
      }();

      if ((typeof _ret === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret)) === "object") return _ret.v;
    }
  }
  function genSpread(ast) {
    var val = ast.arg.val;
    return t.spreadProperty(val);
  }
  function genNode(ast) {
    switch (ast.type) {
      case 'FieldSet':
        return genFieldSet(ast);
      case 'Field':
        return genField(ast);
      case 'Spread':
        return genSpread(ast);
      default:
        throw new Error('Unexpeced node type ' + ast.type);
    }
  }
  return genNode(ast);
}
exports.default = codeGen;